import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/services.dart';
import 'package:logger/logger.dart';
import 'package:moyoung_ble_plugin/impl/channel_names.dart';
import 'package:moyoung_ble_plugin/impl/moyang_beans.dart';
import 'package:path_provider/path_provider.dart';

import 'event_streams.dart';

class MoyoungBleImpl {
  static final MoyoungBleImpl _instance = MoyoungBleImpl();

  /// The default instance of [MoyoungBleImpl] to use.
  ///
  /// Defaults to [MoyoungBleImpl].
  static MoyoungBleImpl get instance => _instance;

  MethodChannel mBleScanMethodChannel =
      const MethodChannel(ChannelNames.METHOD_CHL_BLE_SCAN);
  MethodChannel mConnMethodChannel =
      const MethodChannel(ChannelNames.METHOD_CHL_CONN);

  Logger logger = Logger();

  final _bleEveChannels = BleEventChannels();

  Future<bool> get startScan async {
    return await mBleScanMethodChannel.invokeMethod('startScan');
  }

  Future<String> get cancelScan async {
    return await mBleScanMethodChannel.invokeMethod('cancelScan');
  }

  Future<bool> isConnected(String address) async {
    return await mConnMethodChannel
        .invokeMethod('isConnected', <String, String>{"address": address});
  }

  Future<int> connect(String address) async {
    return await mConnMethodChannel
        .invokeMethod('connect', <String, String>{"address": address});
  }

  Future<bool> disconnect() async {
    return await mConnMethodChannel.invokeMethod('disconnect');
  }

  Future<void> get syncTime async {
    return await mConnMethodChannel.invokeMethod('syncTime');
  }

  Future<void> sendTimeSystem(int timeSystemType) async {
    return await mConnMethodChannel.invokeMethod(
        'sendTimeSystem', <String, int>{"timeSystemType": timeSystemType});
  }

  Future<int> get queryTimeSystem async {
    int time = await mConnMethodChannel.invokeMethod('queryTimeSystem');
    logger.d('queryTimeSystem=' + time.toString());
    return time;
  }

  Future<String> get queryFirmwareVersion async {
    return await mConnMethodChannel.invokeMethod('queryFirmwareVersion');
  }

  Future<void> get queryDeviceBattery async {
    return await mConnMethodChannel.invokeMethod('queryDeviceBattery');
  }

  Future<void> get subscribeDeviceBattery async {
    return await mConnMethodChannel.invokeMethod('subscribeDeviceBattery');
  }

  Future<FirmwareVersionInfo> get checkFirmwareVersion async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('checkFirmwareVersion');
    return firmwareVersionInfoFromJson(jsonStr);
  }

  Future<void> sendUserInfo(CrpUserInfo userInfo) async {
    String jsonStr = crpUserInfoToJson(userInfo);
    return await mConnMethodChannel
        .invokeMethod('sendUserInfo', <String, String>{"userInfo": jsonStr});
  }

  Future<void> sendStepLength(int stepLength) async {
    return await mConnMethodChannel.invokeMethod(
        'sendStepLength', <String, int>{"stepLength": stepLength});
  }

  Future<void> sendTodayWeather(CrpTodayWeatherInfo todayWeatherInfo) async {
    String jsonStr = crpTodayWeatherInfoToJson(todayWeatherInfo);
    return await mConnMethodChannel.invokeMethod(
        'sendTodayWeather', <String, String>{"todayWeatherInfo": jsonStr});
  }

  Future<void> sendFutureWeather(CrpTodayWeatherInfo todayWeatherInfo) async {
    String jsonStr = crpTodayWeatherInfoToJson(todayWeatherInfo);
    return await mConnMethodChannel.invokeMethod(
        'sendFutureWeather', <String, String>{"futureWeatherInfo": jsonStr});
  }

  Stream<BleScanEvent> get bleScanEveStm {
    return _bleEveChannels.bleScanEveStm;
  }

  Stream<int> get connStateEveStm {
    return _bleEveChannels.connStateEveStm;
  }

  Stream<ConnStepChangeBean> get connStepChangeEveStm {
    return _bleEveChannels.connStepChangeEveStm;
  }

  //余诗霞:start
  Future<int> get queryMetricSystem async {
    int metricSystem =
        await mConnMethodChannel.invokeMethod('queryMetricSystem');
    logger.d('queryMetricSystem=' + metricSystem.toString());
    return metricSystem;
  }

  Future<void> sendMetricSystem(int metricSystemType) async {
    await mConnMethodChannel.invokeMethod('sendMetricSystem',
        <String, int>{"metricSystemType": metricSystemType});
  }

  Future<void> sendQuickView(bool quickViewState) async {
    await mConnMethodChannel.invokeMethod(
        'sendQuickView', <String, bool>{"quickViewState": quickViewState});
  }

  Future<bool> get queryQuickView async {
    bool quickViewState =
        await mConnMethodChannel.invokeMethod('queryQuickView');
    logger.d('queryQuickView=' + quickViewState.toString());
    return quickViewState;
  }

  Future<void> sendQuickViewTime(CrpPeriodTimeInfo crpPeriodTimeInfo) async {
    String jsonStr = crpPeriodTimeInfoToJson(crpPeriodTimeInfo);
    await mConnMethodChannel.invokeMethod(
        'sendQuickViewTime', <String, String>{"periodTimeInfo": jsonStr});
  }

  Future<Map<dynamic, dynamic>> get queryQuickViewTime async {
    Map<dynamic, dynamic> jsonStr =
        await mConnMethodChannel.invokeMethod('queryQuickViewTime');
    logger.d('queryQuickViewTime=' + jsonStr.toString());
    return jsonStr;
  }

  Future<void> sendGoalSteps(int goalSteps) async {
    await mConnMethodChannel
        .invokeMethod('sendGoalSteps', <String, int>{"goalSteps": goalSteps});
  }

  Future<int> get queryGoalStep async {
    int goalSteps = await mConnMethodChannel.invokeMethod('queryGoalStep');
    logger.d('queryGoalStep=' + goalSteps.toString());
    return goalSteps;
  }

  Future<void> sendDisplayWatchFace(int watchFaceType) async {
    await mConnMethodChannel.invokeMethod(
        'sendDisplayWatchFace', <String, int>{"watchFaceType": watchFaceType});
  }

  Future<int> get queryDisplayWatchFace async {
    int watchFaceType =
        await mConnMethodChannel.invokeMethod('queryDisplayWatchFace');
    logger.d('queryDisplayWatchFace=' + watchFaceType.toString());
    return watchFaceType;
  }

  Future<CrpWatchFaceLayoutInfo> get queryWatchFaceLayout async {
    String jsonStr = await mConnMethodChannel.invokeMethod('queryWatchFaceLayout');
    CrpWatchFaceLayoutInfo crpWatchFaceLayoutInfo =
        crpWatchFaceLayoutInfoFromJson(jsonStr);
    logger.d('queryWatchFaceLayout=' + jsonStr);
    return crpWatchFaceLayoutInfo;
  }

  Future<void> sendWatchFaceLayout(
      CrpWatchFaceLayoutInfo crpWatchFaceLayoutInfo) async {
    String jsonStr = crpWatchFaceLayoutInfoToJson(crpWatchFaceLayoutInfo);
    await mConnMethodChannel.invokeMethod('sendWatchFaceLayout',
        <String, String>{"watchFaceLayoutInfo": jsonStr});
  }

  Future<Map<dynamic, dynamic>> sendWatchFaceBackground(
      CrpWatchFaceBackgroundInfo watchFaceBackgroundInfo) async {
    String jsonStr = crpWatchFaceBackgroundInfoToJson(watchFaceBackgroundInfo);
    Map<dynamic, dynamic> result = await mConnMethodChannel.invokeMethod(
        'sendWatchFaceBackground',
        <String, String>{"watchFaceBackgroundInfo": jsonStr});
    return result;
  }

  Stream<Map<dynamic, dynamic>> get connLazyFileTransEveStm {
    return _bleEveChannels.connLazyFileTransEveStm;
  }

  Future<CrpSupportWatchFaceInfo> get querySupportWatchFace async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('querySupportWatchFace');
    CrpSupportWatchFaceInfo supportWatchFaceInfo =
        crpSupportWatchFaceInfoFromJson(jsonStr);
    logger.d("querySupportWatchFace=" + jsonStr);
    return supportWatchFaceInfo;
  }

  Future<Map<dynamic, dynamic>> queryWatchFaceStore(
      WatchFaceStoreBean watchFaceStoreBean) async {
    String jsonStr = watchFaceStoreBeanToJson(watchFaceStoreBean);
    logger.d(jsonStr);
    Map<dynamic, dynamic> str = await mConnMethodChannel.invokeMethod(
        'queryWatchFaceStore', <String, String>{"watchFaceStoreBean": jsonStr});
    logger.d("querySupportWatchFace=" + str.toString());
    return str;
  }

  Future<Map<dynamic, dynamic>> queryWatchFaceOfID(int id) async {
    Map<dynamic, dynamic> str = await mConnMethodChannel
        .invokeMethod('queryWatchFaceOfID', <String, int>{"id": id});
    logger.d("queryWatchFaceOfID=" + str.toString());
    return str;
  }

  Future<Map<dynamic, dynamic>> sendWatchFace(
      CrpCustomizeWatchFaceInfo customizeWatchFaceInfo, int timeout) async {
    String pathFile = "";
    await getExternalStorageDirectories().then((value) => value == null
        ? "./"
        : (value.map((e) => pathFile = e.path + customizeWatchFaceInfo.file))
            .toString());
    logger.d("------" + pathFile);
    CrpCustomizeWatchFaceInfo info = CrpCustomizeWatchFaceInfo(
        index: customizeWatchFaceInfo.index, file: pathFile);
    String jsonStr = crpCustomizeWatchFaceInfoToJson(info);
    Map<dynamic, dynamic> result = await mConnMethodChannel.invokeMethod(
        'sendWatchFace',
        <String, Object>{"watchFaceFlutterBean": jsonStr, "timeout": timeout});
    return result;
  }

  Stream<Map<dynamic, dynamic>> get connLazyWFFileTransEveStm {
    return _bleEveChannels.connLazyWFFileTransEveStm;
  }

  Future<void> sendAlarmClock(CrpAlarmClockInfo crpAlarmClockInfo) async {
    String jsonStr = crpAlarmClockInfoToJson(crpAlarmClockInfo);
    await mConnMethodChannel.invokeMethod(
        'sendAlarmClock', <String, String>{"alarmClockInfo": jsonStr});
  }

  Future<List<CrpAlarmClockInfo>> get queryAllAlarmClock async {
    String jsonStr =
        await mConnMethodChannel.invokeMethod('queryAllAlarmClock');
    var listDynamic = jsonDecode(jsonStr);
    List<CrpAlarmClockInfo> listStr = (listDynamic as List<dynamic>)
        .map((e) => CrpAlarmClockInfo.fromJson((e as Map<String, dynamic>)))
        .toList();

    logger.d('queryAllAlarmClock=' + jsonStr);
    return listStr;
  }

  //心率
  Stream<ConnHeartRateBean> get connHeartRateEveStm {
    return _bleEveChannels.connHeartRateEveStm;
  }

  Future<void> get queryLastDynamicRate async {
    await mConnMethodChannel.invokeMethod('queryLastDynamicRate');
  }

  Future<void> enableTimingMeasureHeartRate(int interval) async {
    await mConnMethodChannel.invokeMethod(
        'enableTimingMeasureHeartRate', <String, int>{"interval": interval});
  }

  Future<void> get disableTimingMeasureHeartRate async {
    await mConnMethodChannel.invokeMethod('disableTimingMeasureHeartRate');
  }

  Future<int> get queryTimingMeasureHeartRate async {
    int timeHR =
        await mConnMethodChannel.invokeMethod('queryTimingMeasureHeartRate');
    logger.d('queryTimingMeasureHeartRate=' + timeHR.toString());
    return timeHR;
  }

  Future<void> queryTodayHeartRate(int heartRateType) async {
    await mConnMethodChannel.invokeMethod(
        'queryTodayHeartRate', <String, int>{"heartRateType": heartRateType});
  }

  Future<void> get queryPastHeartRate async {
    await mConnMethodChannel.invokeMethod('queryPastHeartRate');
  }

  Future<void> get queryMovementHeartRate async {
    await mConnMethodChannel.invokeMethod('queryMovementHeartRate');
  }

  Future<void> get startMeasureOnceHeartRate async {
    await mConnMethodChannel.invokeMethod('startMeasureOnceHeartRate');
  }

  Future<void> get stopMeasureOnceHeartRate async {
    await mConnMethodChannel.invokeMethod('stopMeasureOnceHeartRate');
  }

  Future<void> get queryHistoryHeartRate async {
    await mConnMethodChannel.invokeMethod('queryHistoryHeartRate');
  }

  //血压
  Stream<ConnBloodPressureBean> get connBloodPressureEveStm {
    return _bleEveChannels.connBloodPressureEveStm;
  }

  Future<void> get startMeasureBloodPressure async {
    await mConnMethodChannel.invokeMethod('startMeasureBloodPressure');
  }

  Future<void> get stopMeasureBloodPressure async {
    await mConnMethodChannel.invokeMethod('stopMeasureBloodPressure');
  }

  Future<void> get enableContinueBloodPressure async {
    await mConnMethodChannel.invokeMethod('enableContinueBloodPressure');
  }

  Future<void> get disableContinueBloodPressure async {
    await mConnMethodChannel.invokeMethod('disableContinueBloodPressure');
  }

  Future<void> get queryContinueBloodPressureState async {
    await mConnMethodChannel.invokeMethod('queryContinueBloodPressureState');
  }

  Future<void> get queryLast24HourBloodPressure async {
    await mConnMethodChannel.invokeMethod('queryLast24HourBloodPressure');
  }

  Future<void> get queryHistoryBloodPressure async {
    await mConnMethodChannel.invokeMethod('queryHistoryBloodPressure');
  }

  //血氧
  Stream<ConnBloodOxygenBean> get connBloodOxygenEveStm {
    return _bleEveChannels.connBloodOxygenEveStm;
  }

  Future<void> get startMeasureBloodOxygen async {
    await mConnMethodChannel.invokeMethod('startMeasureBloodOxygen');
  }

  Future<void> get stopMeasureBloodOxygen async {
    await mConnMethodChannel.invokeMethod('stopMeasureBloodOxygen');
  }

  Future<void> enableTimingMeasureBloodOxygen(int interval) async {
    await mConnMethodChannel.invokeMethod(
        'enableTimingMeasureBloodOxygen', <String, int>{"interval": interval});
  }

  Future<void> get disableTimingMeasureBloodOxygen async {
    await mConnMethodChannel.invokeMethod('disableTimingMeasureBloodOxygen');
  }

  Future<void> get queryTimingBloodOxygenMeasureState async {
    await mConnMethodChannel.invokeMethod('queryTimingBloodOxygenMeasureState');
  }

  Future<void> queryTimingBloodOxygen(String bloodOxygenTimeType) async {
    await mConnMethodChannel.invokeMethod('queryTimingBloodOxygen',<String, String>{"bloodOxygenTimeType": bloodOxygenTimeType});
  }

  Future<void> get enableContinueBloodOxygen async {
    await mConnMethodChannel.invokeMethod('enableContinueBloodOxygen');
  }

  Future<void> get disableContinueBloodOxygen async {
    await mConnMethodChannel.invokeMethod('disableContinueBloodOxygen');
  }

  Future<void> get queryContinueBloodOxygenState async {
    await mConnMethodChannel.invokeMethod('queryContinueBloodOxygenState');
  }

  Future<void> get queryLast24HourBloodOxygen async {
    await mConnMethodChannel.invokeMethod('queryLast24HourBloodOxygen');
  }

  Future<void> get queryHistoryBloodOxygen async {
    await mConnMethodChannel.invokeMethod('queryHistoryBloodOxygen');
  }

  //拍照
  Future<void> get enterCameraView async {
    await mConnMethodChannel.invokeMethod('enterCameraView');
  }

  Future<void> get exitCameraView async {
    await mConnMethodChannel.invokeMethod('exitCameraView');
  }

  Stream<void> get connCameraEveStm {
    return _bleEveChannels.connCameraEveStm;
  }

  //电话
  Stream<int> get connPhoneEveStm {
    return _bleEveChannels.connPhoneEveStm;
  }

  //rssi
  Stream<int> get connDeviceRssiEveStm {
    return _bleEveChannels.connDeviceRssiEveStm;
  }

  Future<void> get readDeviceRssi async {
    await mConnMethodChannel.invokeMethod('readDeviceRssi');
  }

  //egc
  Future<Map<dynamic, dynamic>> setECGChangeListener(
      String ecgMeasureType) async {
    Map<dynamic, dynamic> result = await mConnMethodChannel.invokeMethod(
        'setECGChangeListener',
        <String, String>{"ecgMeasureType": ecgMeasureType});
    return result;
  }

  Stream<ConnEgcBean> get connLazyEgcEveStm {
    return _bleEveChannels.connLazyEgcEveStm;
  }

  Future<void> get startECGMeasure async {
    await mConnMethodChannel.invokeMethod('startECGMeasure');
  }

  Future<void> get stopECGMeasure async {
    await mConnMethodChannel.invokeMethod('stopECGMeasure');
  }

  Future<void> get isNewECGMeasurementVersion async {
    await mConnMethodChannel.invokeMethod('isNewECGMeasurementVersion');
  }

  Future<void> get queryLastMeasureECGData async {
    await mConnMethodChannel.invokeMethod('queryLastMeasureECGData');
  }

  Future<void> sendECGHeartRate(int heartRate) async {
    await mConnMethodChannel.invokeMethod(
        'sendECGHeartRate', <String, int>{"heartRate": heartRate});
  }

  //以下流是李宝忠的
  Future<Map<dynamic, dynamic>> sendContactAva(
      CrpFileTrans contactAvatarInfo) async {
    String jsonStr = crpFileTransToJson(contactAvatarInfo);
    Map<dynamic, dynamic> result = await mConnMethodChannel.invokeMethod(
        'sendContactAvatar', <String, String>{"contactAvatarInfo": jsonStr});
    return result;
  }

  Stream<Map<dynamic, dynamic>> get connLazyContactAvatarEveStm {
    return _bleEveChannels.connLazyContactAvatarEveStm;
  }

  //余诗霞:end
  //yushixia:end

//xulei:start
  //发送手表版本
  Future<void> sendDeviceLanguage(int language) async {
    await mConnMethodChannel.invokeMethod(
        'sendDeviceLanguage',<String, int>{"language": language});
  }

  Future<Map<dynamic,dynamic>> get queryDeviceLanguage async {
    Map<dynamic,dynamic> jsonStr =
    await mConnMethodChannel.invokeMethod('queryDeviceLanguage');
    logger.d('queryDeviceLanguage=' + jsonStr.toString());
    return jsonStr;
  }

  Future<void> sendOtherMessageState(bool messageState) async{
    await mConnMethodChannel.invokeMethod(
        'sendOtherMessageState',<String,bool>{"messageState":messageState});
  }

  Future<bool> get queryOtherMessageState async{
    bool messageState = await mConnMethodChannel.invokeMethod('queryOtherMessageState');
    logger.d('queryOtherMessageState=',messageState.toString());
    return messageState;
  }

  Future<void> sendMessage(CrpMessageInfo crpMessageInfo) async{
    String jsonStr = crpMessageInfoToJson(crpMessageInfo);
    await mConnMethodChannel.invokeMethod(
        'sendMessage', <String, String>{"messageInfo": jsonStr});
  }

  Future<void> get sendCall0ffHook async{
    await mConnMethodChannel.invokeMethod('sendCall0ffHook');
  }
  // 久坐提醒
  Future<void> sendSedentaryReminder(bool sedentaryReminder) async{
    await mConnMethodChannel.invokeMethod('sendSedentaryReminder',
        <String,bool>{"sedentaryReminder":sedentaryReminder});
  }

  Future<bool> get querySedentaryReminder async{
    bool sedentaryReminder = await mConnMethodChannel.invokeMethod('querySedentaryReminder');
    logger.d('querySedentaryReminder=',sedentaryReminder.toString());
    return sedentaryReminder;
  }

  Future<void> sendSedentaryReminderPeriod(CrpSedentaryReminderPeriodInfo crpSedentaryReminderPeriodInfo) async{
    String jsonStr = crpSedentaryReminderPeriodInfoToJson(crpSedentaryReminderPeriodInfo);
    await mConnMethodChannel.invokeMethod(
        'sendSedentaryReminderPeriod', <String, String>{"crpSedentaryReminderPeriodInfo": jsonStr});
  }

  Future<CrpSedentaryReminderPeriodInfo> get querySedentaryReminderPeriod async {
    String jsonStr =
    await mConnMethodChannel.invokeMethod('querySedentaryReminderPeriod');
    CrpSedentaryReminderPeriodInfo crpSedentaryReminderPeriodInfo=crpSedentaryReminderPeriodInfoFromJson(jsonStr);
    logger.d('querySedentaryReminderPeriod=' + jsonStr);
    return crpSedentaryReminderPeriodInfo;
  }

  //寻找手表
  Future<void> get findDevice async{
    await mConnMethodChannel.invokeMethod('findDevice');
  }

  Future<void> get shutDown async{
    await mConnMethodChannel.invokeMethod('shutDown');
  }

  Future<Map<dynamic,dynamic>> get queryDoNotDistrubTime async{
    Map<dynamic,dynamic> jsonStr =
    await mConnMethodChannel.invokeMethod('queryDoNotDistrubTime');
    // String type = jsonStr['type'];
    logger.d('queryDoNotDistrubTime=' + jsonStr.toString());
    return jsonStr;
  }

  Future<void> sendDoNotDistrubTime(CrpPeriodTimeInfo crpPeriodTimeInfo) async{
    String jsonStr = crpPeriodTimeInfoToJson(crpPeriodTimeInfo);
    await mConnMethodChannel.invokeMethod(
        'sendDoNotDistrubTime', <String, String>{"crpPeriodTimeInfo": jsonStr});
  }

  Future<void> sendBreathingLight(bool breathingLight) async{
    await mConnMethodChannel.invokeMethod('sendBreathingLight',
        <String,bool>{"breathingLight":breathingLight});
  }

  Future<bool> get queryBreathingLight async{
    bool breathingLight = await mConnMethodChannel.invokeMethod('queryBreathingLight');
    logger.d('queryBreathingLight=',breathingLight.toString());
    return breathingLight;
  }

//xulei:end

  /// libaozhong: start
  /// 设置女生生理周期提醒
  Future<void> sendPhysiologcalPeriod(CrpPhysiologcalPeriodInfo info) async {
    String jsonStr = crpPhysiologcalPeriodInfoToJson(info);
    await mConnMethodChannel.invokeMethod(
        "sendPhysiologcalPeriod",
        <String, String>{ "futureWeatherInfo":jsonStr });
  }

  /// 获取女生生理周期提醒
  Future<FirmwareVersionInfo> get queryPhysiologcalPeriod async {
    String resultJson =
        await mConnMethodChannel.invokeMethod("queryPhysiologcalPeriod");
    logger.d(resultJson);
    return firmwareVersionInfoFromJson(resultJson);
  }

  /// 开始找寻手机
  Future<void> get startFindPhone async {
    await mConnMethodChannel.invokeMethod("startFindPhone");
  }

  /// 结束找寻手机
  Future<void> get stopFindPhone async {
    await mConnMethodChannel.invokeMethod("stopFindPhone");
  }

  /// 2.35
  /// 设置播放器状态
  Future<void> get setMusicPlayerState async {
    await mConnMethodChannel.invokeMethod("setMusicPlayerState",
        <String, String>{"CRPMusicPlayerStateType": "1"});
  }

  /// 设置歌的标题
  Future<void> sendSongTitle(String title) async {
    await mConnMethodChannel
        .invokeMethod("sendSongTitle", <String, String>{"title": title});
  }

  /// 设置歌词
  Future<void> sendLyrics(String lyrics) async {
    await mConnMethodChannel
        .invokeMethod("sendLyrics", <String, String>{"lyrics": lyrics});
  }

  /// 关闭音乐控制
  Future<void> get closeMusicControl async {
    await mConnMethodChannel.invokeMethod("closeMusicControl");
  }

  /// 设置当前音量
  Future<void> sendCurrentVolume(int volume) async {
    await mConnMethodChannel
        .invokeMethod("sendCurrentVolume", <String, int>{"volume": volume});
  }

  /// 设置最大音量
  Future<void> sendMaxVolume(int volume) async {
    await mConnMethodChannel
        .invokeMethod("sendMaxVolume", <String, int>{"volume": volume});
  }

  /// 2.36
  /// 喝水提醒
  Future<void> enableDrinkWaterReminder(CrpDrinkWaterPeriodInfo drinkWaterPeriodInfo) async {
    String jsonStr = crpDrinkWaterPeriodInfoToJson(drinkWaterPeriodInfo);
    await mConnMethodChannel.invokeMethod('enableDrinkWaterReminder',
        <String, String>{"drinkWaterPeriodInfo": jsonStr});
  }

  /// 禁用水提醒
  Future<void> get disableDrinkWaterReminder async {
    await mConnMethodChannel.invokeMethod("disableDrinkWaterReminder");
  }

  /// 查询喝水提醒
  Future<String> get queryDrinkWaterReminderPeriod async {
     String jsonStr =  await mConnMethodChannel.invokeMethod("queryDrinkWaterReminderPeriod");
     logger.d(jsonStr);
     return jsonStr;
  }

  /// 2.37
  ///  设置心率告警值
  Future<void> get setMaxHeartRate async {
    await mConnMethodChannel.invokeMethod("setMaxHeartRate");
  }

  /// 查询心跳告警值
  Future<Map<String, dynamic>> get queryMaxHeartRate async {
    Map<String, dynamic> resultMap =
        await mConnMethodChannel.invokeMethod("queryMaxHeartRate");
    logger.d(resultMap);
    return resultMap;
  }

  /// 开始锻炼
  Future<void> startMovement(int i) async {
    await mConnMethodChannel.invokeMethod("startMovement");
  }

  /// 设置锻炼状态
  Future<void> setMovementState(int i) async {
    await mConnMethodChannel.invokeMethod("setMovementState");
  }

  /// 监听锻炼状态
  Stream<String> get connTainStateEveStm {
    return _bleEveChannels.connTainStateEveStm;
  }

  /// 获取协议版本
  Future<String> get getProtocolVersion async {
    String protocolVersion =  await mConnMethodChannel.invokeMethod("getProtocolVersion");
    logger.d(protocolVersion);
    return protocolVersion;
  }

  /// 开始测量第一次温度
  Future<void> get startMeasureTemp async {
    await mConnMethodChannel.invokeMethod("startMeasureTemp");
  }

  /// 温度一次停止测量
  Future<void> get stopMeasureTemp async {
    await mConnMethodChannel.invokeMethod("stopMeasureTemp");
  }

  /// 开启定时温度测量
  Future<void> get enableTimingMeasureTemp async {
    await mConnMethodChannel.invokeMethod("enableTimingMeasureTemp");
  }

  /// 禁用定时温度测量
  Future<void> get disableTimingMeasureTemp async {
    return await mConnMethodChannel.invokeMethod("disableTimingMeasureTemp");
  }

  /// 使用实例查询温度检测状态定时开关
  Future<String> get queryTimingMeasureTempState async {
    String tempState = await mConnMethodChannel.invokeMethod("queryTimingMeasureTempState");
    logger.d(tempState);
    return tempState;
  }

  /// 查询定时测温结果
  Future<void> get queryTimingMeasureTemp async {
    await mConnMethodChannel.invokeMethod("queryTimingMeasureTemp");
  }

  /// 设置温度测量结果监听器
  Stream<String> get connTempChangeEveStm {
    return _bleEveChannels.connTempChangeEveStm;
  }

  /// 设置显示的时间
  Future<void> sendDisplayTime(int time) async {
    await mConnMethodChannel.invokeMethod("sendDisplayTime", <String, int>{ "time":time });
  }

  /// 查询显示时间
  Future<int> get queryDisplayTime async {
    int time = await mConnMethodChannel.invokeMethod("queryDisplayTime");
    logger.d(time);
    return time;
  }

  /// 开启洗手提醒功能
  Future<void> enableHandWashingReminder(CrpHandWashingPeriodInfo handWashingPeriodInfo) async{
    String jsonStr = crpHandWashingPeriodInfoToJson(handWashingPeriodInfo);
    await mConnMethodChannel.invokeMethod("enableHandWashingReminder",
        <String, String>{ "handWashingPeriodInfo":jsonStr });
  }

  /// 禁用洗手提醒
  Future<void> get disableHandWashingReminder async {
    await mConnMethodChannel.invokeMethod("disableHandWashingReminder");
  }

  /// 查询洗手提醒
  Future<int> get queryHandWashingReminderPeriod async {
    int period = await mConnMethodChannel.invokeMethod("queryHandWashingReminderPeriod");
    logger.d(period);
    return period;
  }

  /// 设置本地城市
  Future<void> sendLocalCity(String city) async {
    await mConnMethodChannel.invokeMethod("sendLocalCity",<String, String>{"city": city});
  }

  /// 切换手链的温度系统
  Future<void> sendTempUnit(int temp) async {
    await mConnMethodChannel.invokeMethod("sendTempUnit", <String, int>{"temp": temp});
  }

  /// 查询系统温度
  Future<void> get queryTempUnit async {
    await mConnMethodChannel.invokeMethod("queryTempUnit");
  }

  /// 设置亮度
  Future<void> sendBrightness(int brightness) async {
    await mConnMethodChannel.invokeMethod("sendBrightness", <String, int>{"brightness": brightness});
  }

  /// 获取亮度
  Future<Map<dynamic, dynamic>> get queryBrightness async{
    Map<dynamic, dynamic> brightness = await mConnMethodChannel.invokeMethod("queryBrightness");
    logger.d(brightness);
    return brightness;
  }

  /// 获取经典的蓝牙地址
  Future<String> get queryBtAddress async {
    String address = await mConnMethodChannel.invokeMethod("queryBtAddress");
    logger.d(address);
    return address;
  }

  /// 检查支持联系人
  Future<String> get checkSupportQuickContact async {
     String contactConfigInfo = await mConnMethodChannel.invokeMethod("checkSupportQuickContact");
     logger.d(contactConfigInfo);
     return contactConfigInfo;
  }

  /// 查询当前联系人数
  Future<int> get queryContactCount async {
    int count = await mConnMethodChannel.invokeMethod("queryContactCount");
    logger.d(count);
    return count;
  }

  /// 设置联系人侦听器
  Stream<int> get connContactEveStm {
    return _bleEveChannels.connContactEveStm;
  }

  /// 设置联系人
  Future<void> sendContact(CrpContactInfo info) async {
    String jsonStr = crpContactInfoToJson(info);
    await mConnMethodChannel.invokeMethod("sendContact",<String, String>{"contactBean": jsonStr});
  }

  /// 删除联系人
  Future<void> deleteContact(int id) async {
    await mConnMethodChannel.invokeMethod("deleteContact", <String, int>{"id": id});
  }

  /// 设置省电监听器
  Stream<bool> get connBatterySavingEveStm {
    return _bleEveChannels.connBatterySavingEveStm;
  }

  /// 设置省电状态
  Future<void> sendBatterySaving(bool enable) async {
    logger.d(enable);
    await mConnMethodChannel.invokeMethod("sendBatterySaving", <String, bool>{"enable": enable});
  }

  /// 获取省电状态
  Future<void> get queryBatterySaving async {
    await mConnMethodChannel.invokeMethod("queryBatterySaving");
  }

  /// 查询支持药丸提醒
  Future<String> get queryPillReminder async {
    String jsonStr = await mConnMethodChannel.invokeMethod("queryPillReminder");
    logger.d(jsonStr);
    return jsonStr;
  }

  /// 设置药丸提醒
  Future<void> sendPillReminder(CrpPillReminderInfo info) async {
    String jsonStr = crpPillReminderInfoToJson(info);
    await mConnMethodChannel.invokeMethod("sendPillReminder", <String, String>{"pillReminderInfo": jsonStr});
  }

  /// 删除药丸提醒
  Future<void> deletePillReminder(int id) async {
    await mConnMethodChannel.invokeMethod("deletePillReminder", <String, int>{"id": id});
  }

  /// 清晰的药丸提醒
  Future<void> get clearPillReminder async {
    await mConnMethodChannel.invokeMethod("clearPillReminder");
  }

  /// 查询轻按，进入唤醒状态
  Future<bool> get queryTapToWakeState async {
    bool state = await mConnMethodChannel.invokeMethod("queryTapToWakeState");
    logger.d(state);
    return state;
  }

  /// 设置“轻按”为唤醒状态
  Future<void> sendTapToWakeState(bool enable) async {
    await mConnMethodChannel.invokeMethod("sendTapToWakeState", <String, bool>{"enable": enable});
  }

  /// 设置培训侦听器
  Stream<String> get connTrainingEveStm {
    return _bleEveChannels.connTrainingEveStm;
  }

  /// 查询历史训练
  Future<void> get queryHistoryTraining async {
    await mConnMethodChannel.invokeMethod("queryHistoryTraining");
  }

  /// 查询培训细节
  Future<void> queryTraining(int id) async {
    await mConnMethodChannel.invokeMethod("queryTraining", <String, int>{"id": id});
  }
  /// libaozhong: end

}
